#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 IAS / CNRS / Univ. Paris-Sud
# BSD License - see attached LICENSE file
# Author: Alexandre Boucaud <alexandre.boucaud@ias.u-psud.fr>
r"""
 ____        ____  _   _ _____ ____
|  _ \ _   _|  _ \| | | | ____|  _ \
| |_) | | | | |_) | |_| |  _| | |_) |
|  __/| |_| |  __/|  _  | |___|  _ <
|_|    \__, |_|   |_| |_|_____|_| \_\
       |___/
---
Python-based PSF Homogenization kERnels
"""



# -*- coding: utf-8 -*-

# Copyright (c) 2016 IAS / CNRS / Univ. Paris-Sud
# BSD License - see attached LICENSE file
# Author: Alexandre Boucaud <alexandre.boucaud@ias.u-psud.fr>

from __future__ import division

import os
import tempfile
import collections

import pytest
import numpy as np
import astropy.io.fits as fits


def create_mock_fits():
    x = np.ones((5, 5))
    prihdu = fits.PrimaryHDU(x)

    # Single extension FITS
    img = fits.ImageHDU(data=x)
    singlehdu = fits.HDUList([prihdu, img])
    singlehdu.writeto('image.fits', overwrite=True)


@pytest.fixture(scope="module")
def fitscleandir():
    newpath = tempfile.mkdtemp()
    os.chdir(newpath)
    create_mock_fits()


@pytest.fixture
def mock_parser():
    MockParser = collections.namedtuple('MockParser',
                                        ['psf_source', 'psf_target', 'output',
                                         'angle_source', 'angle_target',
                                         'reg_fact'])
    parser = MockParser(psf_source='psf_source.fits',
                        psf_target='psf_target.fits',
                        output='output.fits',
                        angle_source=0.0,
                        angle_target=0.0,
                        reg_fact=1.e-4)
    return parser


@pytest.fixture(scope='function', params=[15, 55, 255])
def tones(request):
    size = request.param
    return np.ones((size, size)), size


@pytest.fixture(scope='function', params=[0.11, 0.22, 0.33])
def trange(request):
    step = request.param
    return np.arange(0, 10, step)


@pytest.fixture(scope='function', params=[5, 10])
def floatrange(request):
    return request.param


@pytest.fixture(scope='function', params=[3, 15, 101])
def imagerot(request):
    size = request.param
    array = np.zeros((size, size))
    array[0, size//2] = 1
    return array


@pytest.fixture(scope='function', params=[3, 15, 100])
def imagedirac(request):
    size = request.param
    array = np.zeros((size, size))
    array[size // 2, size // 2] = 1
    return array



# -*- coding: utf-8 -*-

# Copyright (c) 2016 IAS / CNRS / Univ. Paris-Sud
# BSD License - see attached LICENSE file
# Author: Alexandre Boucaud <alexandre.boucaud@ias.u-psud.fr>

from __future__ import division, absolute_import

import pytest
import numpy as np
import astropy.io.fits as fits

from numpy.testing import assert_equal, assert_allclose

from pypher.pypher import (parse_args, format_kernel_header,
                           imrotate, imresample, trim, zero_pad,
                           psf2otf, homogenization_kernel)
from pypher.fitsutils import has_pixelscale, get_pixscale, add_comments
from pypher.parser import ArgumentParserError
from pypher.addpixscl import parse_args as parse_args_addpixscl

ERRSHAPE = 'incorrect shape'
ERROUT = 'incorrect output'
ERRVAL = 'incorrect value'
PIXSCALE = 1.0
ABSTOL = 1e-6
RELTOL = 1e-6


class TestParser(object):
    def test_parse_args(self):
        with pytest.raises(ArgumentParserError):
            parse_args()

    def test_parse_args_addpixscl(self):
        with pytest.raises(ArgumentParserError):
            parse_args_addpixscl()


class TestFits(object):
    def test_nopixelscale(self, fitscleandir):
        with pytest.raises(IOError):
            get_pixscale('image.fits')

    def test_format_header(self, mock_parser):
        format_kernel_header('image.fits', mock_parser, PIXSCALE)
        assert round(fits.getval('image.fits', 'CD1_1') * 3600, 1) == PIXSCALE
        assert round(fits.getval('image.fits', 'CD1_2') * 3600, 1) == 0.0
        assert round(fits.getval('image.fits', 'CD2_1') * 3600, 1) == 0.0
        assert round(fits.getval('image.fits', 'CD2_2') * 3600, 1) == PIXSCALE

    def test_has_pixelscale(self):
        assert has_pixelscale('image.fits')

    def test_get_pixelscale(self):
        pscale = get_pixscale('image.fits')
        assert round(pscale, 1) == PIXSCALE

    def test_add_single_comment(self):
        add_comments('image.fits', "single comment")
        comments = str(fits.getval('image.fits', 'COMMENT')).split('\n')
        assert comments[-1] == "single comment"


class TestImageManipulation(object):
    def test_rotate90(self, imagerot):
        assert_allclose(imrotate(imagerot, 90), np.rot90(imagerot, 3),
                        atol=ABSTOL, rtol=RELTOL)

    def test_rotate180(self, imagerot):
        assert_allclose(imrotate(imagerot, 180), np.rot90(imagerot, 2),
                        atol=ABSTOL, rtol=RELTOL)

    def test_resample_even(self):
        size = 42
        factor = 2
        res = imresample(np.ones((size, size)),
                         source_pscale=factor,
                         target_pscale=1)
        assert res.shape[0] == size * factor

    def test_resample_odd(self):
        size = 49
        factor = 2
        res = imresample(np.ones((size, size)),
                         source_pscale=factor,
                         target_pscale=1)
        assert res.shape[0] == size * factor + 1

    def test_resample_memoryerror(self):
        with pytest.raises(MemoryError):
            imresample(np.zeros((200, 200)), 100, 1)

    def test_trim(self, tones):
        arr, size = tones
        size_e = size - 1
        size_o = size - 2
        shape_ee = (size_e, size_e)
        shape_eo = (size_e, size_o)
        shape_oo = (size_o, size_o)

        assert_equal(trim(arr, (size, size)), arr, ERROUT)
        # Shapes
        assert_equal(trim(arr, shape_oo).shape,
                     shape_oo,
                     ERRSHAPE)
        assert_equal(trim(arr, np.asarray(shape_oo, float)).shape,
                     shape_oo,
                     ERRSHAPE)
        # Wrong inputs
        with pytest.raises(ValueError):
            trim(arr, 0)
            trim(arr, (size_e, 0))
            trim(arr, (size + 1, size))
            trim(arr, shape_ee)
            trim(arr, shape_eo)

    def test_zero_pad(self, tones):
        arr, size = tones
        size_e = size + 1
        size_o = size + 2
        shape_ee = (size_e, size_e)
        shape_eo = (size_e, size_o)
        shape_oo = (size_o, size_o)

        assert_equal(zero_pad(arr, (size, size)), arr, ERROUT)
        # Shapes
        for pos in ['corner', 'center']:
            assert_equal(zero_pad(arr, shape_ee, 'corner').shape,
                         shape_ee,
                         ERRSHAPE)
            assert_equal(zero_pad(arr, shape_oo, pos).shape,
                         shape_oo,
                         ERRSHAPE)
            assert_equal(zero_pad(arr, shape_eo, 'corner').shape,
                         shape_eo,
                         ERRSHAPE)
            # Wrong inputs
            with pytest.raises(ValueError):
                zero_pad(arr, -2, position=pos)
                zero_pad(arr, size_e, position=pos)
                zero_pad(arr, (size - 1, size_e), pos)
                zero_pad(arr, shape_ee, 'center')
                zero_pad(arr, shape_eo, 'center')


class TestFourier(object):
    def test_dirac_otf(self, imagedirac):
        shape = imagedirac.shape
        assert_equal(psf2otf(imagedirac, shape), np.ones(shape))

    def test_homogenization_dtype(self, imagedirac):
        center = imagedirac.shape[0] // 2
        target = np.zeros_like(imagedirac)
        s = slice(center - 1, center + 2)
        target[s, s] += 1.
        target[center, center] += 1
        target /= target.sum()

        k, kf = homogenization_kernel(target, imagedirac,
                                      reg_fact=1e-4, clip=True)

        assert k.dtype == float
        assert kf.dtype == complex



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 IAS / CNRS / Univ. Paris-Sud
# BSD License - see attached LICENSE file
# Author: Alexandre Boucaud <alexandre.boucaud@ias.u-psud.fr>

"""
fitsutils.py
------------
A set of convenience methods to deal with FITS files

"""
from __future__ import absolute_import, print_function, division

import astropy.io.fits as pyfits
from astropy.io.fits import getdata, writeto


PIXSCL_KEY_DEG = ['CD1_1', 'CD2_2', 'CDELT1', 'CDELT2']
PIXSCL_KEY_ARCSEC = ['PIXSCALE', 'SECPIX', 'PIXSCALX', 'PIXSCALY']
PIXSCL_KEYS = PIXSCL_KEY_DEG + PIXSCL_KEY_ARCSEC


def has_pixelscale(fits_file):
    """
    Find pixel scale keywords in FITS file

    Parameters
    ----------
    fits_file: str
        Path to a FITS image file

    """
    header = pyfits.getheader(fits_file)
    return [key
            for key in PIXSCL_KEYS
            if key in list(header.keys())]


def write_pixelscale(fits_file, value, ext=0):
    """
    Write pixel scale information to a FITS file header

    The input pixel scale value is given in arcseconds but is stored
    in degrees since the chosen header KEYS are the linear
    transformation matrix parameters CDi_ja

    Parameters
    ----------
    fits_file: str
        Path to a FITS image file
    value: float
        Pixel scale value in arcseconds
    ext: int, optional
        Extension number in the FITS file

    """
    pixscl = value / 3600
    comment = 'Linear transformation matrix'

    pyfits.setval(fits_file, 'CD1_1', value=pixscl, ext=ext, comment=comment)
    pyfits.setval(fits_file, 'CD1_2', value=0.0, ext=ext, comment=comment)
    pyfits.setval(fits_file, 'CD2_1', value=0.0, ext=ext, comment=comment)
    pyfits.setval(fits_file, 'CD2_2', value=pixscl, ext=ext, comment=comment)


def get_pixscale(fits_file):
    """
    Retreive the image pixel scale from its FITS header

    Parameters
    ----------
    fits_file: str
        Path to a FITS image file

    Returns
    -------
    pixel_scale: float
        The pixel scale of the image in arcseconds

    """
    pixel_keys = has_pixelscale(fits_file)

    if not pixel_keys:
        raise IOError("Pixel scale not found in {0}.".format(fits_file))

    pixel_key = pixel_keys.pop()
    pixel_scale = abs(pyfits.getval(fits_file, pixel_key))

    if pixel_key in PIXSCL_KEY_DEG:
        pixel_scale *= 3600

    return round(pixel_scale, 6)


def clear_comments(fits_file):
    """
    Delete the COMMENTS in the FITS header

    Parameters
    ----------
    fits_file: str
        Path to a FITS image file

    """
    for comment_key in ['COMMENT', 'comment', 'Comment']:
        try:
            pyfits.delval(fits_file, comment_key)
        except KeyError:
            pass


def add_comments(fits_file, values):
    """
    Add comments to the FITS header

    Parameters
    ----------
    fits_file: str
        Path to a FITS image file
    values: str or str list
        Comment(s) to add

    """
    if isinstance(values, str):
        pyfits.setval(fits_file, 'COMMENT', value=values)
    else:
        for value in values:
            pyfits.setval(fits_file, 'COMMENT', value=value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 IAS / CNRS / Univ. Paris-Sud
# BSD License - see attached LICENSE file
# Author: Alexandre Boucaud <alexandre.boucaud@ias.u-psud.fr>

"""
addpixscl
---------
Write the pixel scale in FITS file headers

Usage:
  addpixscl fits_files pixel_scale [--ext EXT]
  addpixscl (-h | --help)

Example:
  addpixscl psf_*.fits 0.1
"""
from __future__ import absolute_import, print_function, division

import sys

from pypher.parser import ThrowingArgumentParser, ArgumentParserError
from pypher.fitsutils import has_pixelscale, write_pixelscale


def parse_args():
    """Argument parser for the command line interface of `addpixscl`"""
    parser = ThrowingArgumentParser(
        description='Write the pixel scale in FITS file headers')

    parser.add_argument('fits_files', nargs='+', type=str,
                        help='Name of FITS files')

    parser.add_argument('pixel_scale', type=float,
                        help='Pixel scale value in arcseconds')

    parser.add_argument('-e', '--ext', type=int, default=0,
                        help='FITS extension number')

    return parser.parse_args()


def main():  # pragma: no cover
    """Main script for addpixscl"""
    try:
        args = parse_args()
    except ArgumentParserError:
        print(__doc__)
        sys.exit()

    for fits_file in args.fits_files:
        if has_pixelscale(fits_file):
            print("Found keywords refering to the pixel scale "
                  "in {0} header.".format(fits_file))
            continue

        write_pixelscale(fits_file, args.pixel_scale, args.ext)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 IAS / CNRS / Univ. Paris-Sud
# BSD License - see attached LICENSE file
# Author: Alexandre Boucaud <alexandre.boucaud@ias.u-psud.fr>

"""
PyPHER - Python-based PSF Homogenization kERnels
================================================

Compute the homogenization kernel between two PSFs

Usage:
  pypher psf_source psf_target output
         [-s ANGLE_SOURCE] [-t ANGLE_TARGET] [-r REG_FACT]
  pypher (-h | --help)

Example:
  pypher psf_a.fits psf_b.fits kernel_a_to_b.fits -r 1.e-5
"""
from __future__ import absolute_import, print_function, division

import os
import sys
import logging
import logging.handlers
import argparse
import numpy as np

from scipy.ndimage import rotate, zoom

from pypher import fitsutils as fits
from pypher.parser import ThrowingArgumentParser, ArgumentParserError

__version__ = '0.7.2'


def parse_args():
    """Argument parser for the command line interface of `pypher`"""
    parser = ThrowingArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        prog='pypher',
        description="Compute the homogenization kernel between two PSFs")

    parser.add_argument('psf_source', type=str,
                        help="FITS file of PSF image with highest resolution")

    parser.add_argument('psf_target', type=str,
                        help="FITS file of PSF image with lowest resolution")

    parser.add_argument('output', type=str,
                        help="File name for the output kernel")

    parser.add_argument('-s', '--angle_source', type=float, default=0.0,
                        help="Rotation angle to apply to `psf_source` (deg)")

    parser.add_argument('-t', '--angle_target', type=float, default=0.0,
                        help="Rotation angle to apply to `psf_target` (deg)")

    parser.add_argument('-r', '--reg_fact', type=float, default=1.e-4,
                        help="Regularisation parameter for the Wiener filter")

    return parser.parse_args()

################
# IMAGE METHODS
################


def format_kernel_header(fits_file, args, pixel_scale):
    """
    Write the input parameters of pypher as comments in the header

    The kernel header therefore contains the name of the PSF files
    it has been created from.
    The pixel scale of the kernel is also written as a dedicated
    kernel key.

    Parameters
    ----------
    fits_file: str
        Path to the FITS kernel image
    args: `argparse.Namespace`
        Container for the parsed values
    pixel_scale: float
        Pixel scale of the kernel

    """
    fits.clear_comments(fits_file)

    pypher_comments = [
        '=' * 50, '',
        'File written with PyPHER',
        '------------------------', '',
        'Kernel from PSF', '',
        '=> {0}'.format(os.path.basename(args.psf_source)), '',
        'to PSF', '',
        '=> {0}'.format(os.path.basename(args.psf_target)), '',
        'using a regularisation parameter '
        'R = {0:1.1e}'.format(args.reg_fact), '',
        '=' * 50
    ]
    fits.add_comments(fits_file, pypher_comments)

    fits.write_pixelscale(fits_file, pixel_scale)


def imrotate(image, angle, interp_order=1):
    """
    Rotate an image from North to East given an angle in degrees

    Parameters
    ----------
    image : `numpy.ndarray`
        Input data array
    angle : float
        Angle in degrees
    interp_order : int, optional
        Spline interpolation order [0, 5] (default 1: linear)

    Returns
    -------
    output : `numpy.ndarray`
        Rotated data array

    """
    return rotate(image, -1.0 * angle,
                  order=interp_order, reshape=False, prefilter=False)


def imresample(image, source_pscale, target_pscale, interp_order=1):
    """
    Resample data array from one pixel scale to another

    The resampling ensures the parity of the image is conserved
    to preserve the centering.

    Parameters
    ----------
    image : `numpy.ndarray`
        Input data array
    source_pscale : float
        Pixel scale of ``image`` in arcseconds
    target_pscale : float
        Pixel scale of output array in arcseconds
    interp_order : int, optional
        Spline interpolation order [0, 5] (default 1: linear)

    Returns
    -------
    output : `numpy.ndarray`
        Resampled data array

    """
    old_size = image.shape[0]
    new_size_raw = old_size * source_pscale / target_pscale
    new_size = int(np.ceil(new_size_raw))

    if new_size > 10000:
        raise MemoryError("The resampling will yield a too large image. "
                          "Please resize the input PSF image.")

    # Chech for parity
    if (old_size - new_size) % 2 == 1:
        new_size += 1

    ratio = new_size / old_size

    return zoom(image, ratio, order=interp_order) / ratio**2


def trim(image, shape):
    """
    Trim image to a given shape

    Parameters
    ----------
    image: 2D `numpy.ndarray`
        Input image
    shape: tuple of int
        Desired output shape of the image

    Returns
    -------
    new_image: 2D `numpy.ndarray`
        Input image trimmed

    """
    shape = np.asarray(shape, dtype=int)
    imshape = np.asarray(image.shape, dtype=int)

    if np.all(imshape == shape):
        return image

    if np.any(shape <= 0):
        raise ValueError("TRIM: null or negative shape given")

    dshape = imshape - shape
    if np.any(dshape < 0):
        raise ValueError("TRIM: target size bigger than source one")

    if np.any(dshape % 2 != 0):
        raise ValueError("TRIM: source and target shapes "
                         "have different parity")

    idx, idy = np.indices(shape)
    offx, offy = dshape // 2

    return image[idx + offx, idy + offy]


def zero_pad(image, shape, position='corner'):
    """
    Extends image to a certain size with zeros

    Parameters
    ----------
    image: real 2d `numpy.ndarray`
        Input image
    shape: tuple of int
        Desired output shape of the image
    position : str, optional
        The position of the input image in the output one:
            * 'corner'
                top-left corner (default)
            * 'center'
                centered

    Returns
    -------
    padded_img: real `numpy.ndarray`
        The zero-padded image

    """
    shape = np.asarray(shape, dtype=int)
    imshape = np.asarray(image.shape, dtype=int)

    if np.all(imshape == shape):
        return image

    if np.any(shape <= 0):
        raise ValueError("ZERO_PAD: null or negative shape given")

    dshape = shape - imshape
    if np.any(dshape < 0):
        raise ValueError("ZERO_PAD: target size smaller than source one")

    pad_img = np.zeros(shape, dtype=image.dtype)

    idx, idy = np.indices(imshape)

    if position == 'center':
        if np.any(dshape % 2 != 0):
            raise ValueError("ZERO_PAD: source and target shapes "
                             "have different parity.")
        offx, offy = dshape // 2
    else:
        offx, offy = (0, 0)

    pad_img[idx + offx, idy + offy] = image

    return pad_img


##########
# FOURIER
##########


def udft2(image):
    """Unitary fft2"""
    norm = np.sqrt(image.size)
    return np.fft.fft2(image) / norm


def uidft2(image):
    """Unitary ifft2"""
    norm = np.sqrt(image.size)
    return np.fft.ifft2(image) * norm


def psf2otf(psf, shape):
    """
    Convert point-spread function to optical transfer function.

    Compute the Fast Fourier Transform (FFT) of the point-spread
    function (PSF) array and creates the optical transfer function (OTF)
    array that is not influenced by the PSF off-centering.
    By default, the OTF array is the same size as the PSF array.

    To ensure that the OTF is not altered due to PSF off-centering, PSF2OTF
    post-pads the PSF array (down or to the right) with zeros to match
    dimensions specified in OUTSIZE, then circularly shifts the values of
    the PSF array up (or to the left) until the central pixel reaches (1,1)
    position.

    Parameters
    ----------
    psf : `numpy.ndarray`
        PSF array
    shape : int
        Output shape of the OTF array

    Returns
    -------
    otf : `numpy.ndarray`
        OTF array

    Notes
    -----
    Adapted from MATLAB psf2otf function

    """
    if np.all(psf == 0):
        return np.zeros_like(psf)

    inshape = psf.shape
    # Pad the PSF to outsize
    psf = zero_pad(psf, shape, position='corner')

    # Circularly shift OTF so that the 'center' of the PSF is
    # [0,0] element of the array
    for axis, axis_size in enumerate(inshape):
        psf = np.roll(psf, -int(axis_size / 2), axis=axis)

    # Compute the OTF
    otf = np.fft.fft2(psf)

    # Estimate the rough number of operations involved in the FFT
    # and discard the PSF imaginary part if within roundoff error
    # roundoff error  = machine epsilon = sys.float_info.epsilon
    # or np.finfo().eps
    n_ops = np.sum(psf.size * np.log2(psf.shape))
    otf = np.real_if_close(otf, tol=n_ops)

    return otf


################
# DECONVOLUTION
################

LAPLACIAN = np.array([[ 0, -1,  0],
                      [-1,  4, -1],
                      [ 0, -1,  0]])


def deconv_wiener(psf, reg_fact):
    r"""
    Create a Wiener filter using a PSF image

    The signal is $\ell_2$ penalized by a 2D Laplacian operator that
    serves as a high-pass filter for the regularization process.
    The key to the process is to use optical transfer functions (OTF)
    instead of simple Fourier transform, since it ensures the phase
    of the psf is adequately placed.

    Parameters
    ----------
    psf: `numpy.ndarray`
        PSF array
    reg_fact: float
        Regularisation parameter for the Wiener filter

    Returns
    -------
    wiener: complex `numpy.ndarray`
        Fourier space Wiener filter

    """
    # Optical transfer functions
    trans_func = psf2otf(psf, psf.shape)
    reg_op = psf2otf(LAPLACIAN, psf.shape)

    wiener = np.conj(trans_func) / (np.abs(trans_func)**2 +
                                    reg_fact * np.abs(reg_op)**2)

    return wiener


def homogenization_kernel(psf_target, psf_source, reg_fact=1e-4, clip=True):
    r"""
    Compute the homogenization kernel to match two PSFs

    The deconvolution step is done using a Wiener filter with $\ell_2$
    penalization.
    The output is given both in Fourier and in the image domain to serve
    different purposes.

    Parameters
    ----------
    psf_target: `numpy.ndarray`
        2D array
    psf_source: `numpy.ndarray`
        2D array
    reg_fact: float, optional
        Regularisation parameter for the Wiener filter
    clip: bool, optional
        If `True`, enforces the non-amplification of the noise
        (default `True`)

    Returns
    -------
    kernel_image: `numpy.ndarray`
        2D deconvolved image
    kernel_fourier: `numpy.ndarray`
        2D discrete Fourier transform of deconvolved image

    """
    wiener = deconv_wiener(psf_source, reg_fact)

    kernel_fourier = wiener * udft2(psf_target)
    kernel_image = np.real(uidft2(kernel_fourier))

    if clip:
        kernel_image.clip(-1, 1)

    return kernel_image, kernel_fourier


########
# DEBUG
########


def setup_logger(log_filename='pypher.log'):  # pragma: no cover
    """
    Set up and return a logger

    The logger records the time, modulename, method and message

    Parameters
    ----------
    log_filename: str
        Name of the output logfile

    """
    # create logger
    logger = logging.getLogger('logger')
    logger.setLevel(logging.DEBUG)
    # Add the log message handler to the logger
    handler = logging.handlers.RotatingFileHandler(log_filename)
    # create formatter
    formatter = logging.Formatter('%(asctime)s - '
                                  '%(module)s - '
                                  '%(levelname)s - '
                                  '%(message)s')
    handler.setFormatter(formatter)
    # add handler to logger
    logger.addHandler(handler)

    return logger


#######
# MAIN
#######


def main():  # pragma: no cover
    """Main script for pypher"""
    try:
        args = parse_args()
    except ArgumentParserError:
        print(__doc__)
        sys.exit()

    kernel_basename, _ = os.path.splitext(args.output)
    kernel_fits = kernel_basename + '.fits'

    logname = '%s.log' % kernel_basename
    if os.path.exists(logname):
        os.remove(logname)
    log = setup_logger(logname)

    # Load images (NaNs are set to 0)
    psf_source = fits.getdata(args.psf_source)
    psf_target = fits.getdata(args.psf_target)

    log.info('Source PSF loaded: %s', args.psf_source)
    log.info('Target PSF loaded: %s', args.psf_target)

    # Set NaNs to 0.0
    psf_source = np.nan_to_num(psf_source)
    psf_target = np.nan_to_num(psf_target)

    # Retrieve the pixel scale of each image
    pixscale_source = fits.get_pixscale(args.psf_source)
    pixscale_target = fits.get_pixscale(args.psf_target)

    log.info('Source PSF pixel scale: %.2f arcsec', pixscale_source)
    log.info('Target PSF pixel scale: %.2f arcsec', pixscale_target)

    # Rotate images (if necessary)
    if args.angle_source != 0.0:
        psf_source = imrotate(psf_source, args.angle_source)
    if args.angle_target != 0.0:
        psf_target = imrotate(psf_target, args.angle_target)

    log.info('Source PSF rotated by %.2f degrees', args.angle_source)
    log.info('Target PSF rotated by %.2f degrees', args.angle_target)

    # Normalize the PSFs
    psf_source /= psf_source.sum()
    psf_target /= psf_target.sum()

    # Resample high resolution image to the low one
    if pixscale_source != pixscale_target:
        try:
            psf_source = imresample(psf_source,
                                    pixscale_source,
                                    pixscale_target)
        except MemoryError:
            log.error('- COMPUTATION ABORTED -')
            log.error('The size of the resampled PSF would have '
                      'exceeded 10K x 10K')
            log.error('Please resize your image and try again')

            print('Issue during the resampling step - see pypher.log')
            sys.exit()

        log.info('Source PSF resampled to the target pixel scale')

    # check the new size of the source vs. the target
    if psf_source.shape > psf_target.shape:
        psf_source = trim(psf_source, psf_target.shape)
    else:
        psf_source = zero_pad(psf_source, psf_target.shape, position='center')

    kernel, _ = homogenization_kernel(psf_target, psf_source,
                                      reg_fact=args.reg_fact)

    log.info('Kernel computed using Wiener filtering and a regularisation '
             'parameter r = %.2e', args.reg_fact)

    # Write kernel to FITS file
    fits.writeto(kernel_fits, data=kernel)
    format_kernel_header(kernel_fits, args, pixscale_target)

    log.info('Kernel saved in %s', kernel_fits)

    print("pypher: Output kernel saved to %s" % kernel_fits)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 IAS / CNRS / Univ. Paris-Sud
# BSD License - see attached LICENSE file
# Author: Alexandre Boucaud <alexandre.boucaud@ias.u-psud.fr>
from setuptools import setup, find_packages


def find_version(filepath):
    """
    Find project version in a given file

    The syntax for the file version need to be in the form
    __version__ = 'a.b.c'
    which follows the semantic versioning http://semver.org/
    * a : major version
    * b : minor version
    * c : patch version

    Parameters
    ----------
    filepath: str
        Path to the file containing a version number

    Returns
    -------
    version: str
        The program version in the form 'a.b.c' as described above

    """
    with open(filepath) as pfile:
        for line in pfile.readlines():
            if line.startswith('__version__'):
                version = line.strip()[-6:-1]
    return version

setup(
    name='pypher',
    author='Alexandre Boucaud',
    author_email='alexandre.boucaud@apc.in2p3.fr',
    description='Python-based PSF Homogenization kERnels production',
    license='New BSD',
    url='http://pypher.readthedocs.org/en/latest/',
    download_url='https://github.com/aboucaud/pypher',
    version=find_version('pypher/pypher.py'),
    long_description=open('README.rst').read(),
    long_description_content_type='text/x-rst',
    zip_safe=False,
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'pypher = pypher.pypher:main',
            'addpixscl = pypher.addpixscl:main',
        ],
    },
    install_requires=[
        'numpy>=1.7.2',
        'scipy>=0.9',
        'astropy>=2.0'
    ],
    classifiers=[
        'Programming Language :: Python',
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: BSD License',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Astronomy',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 IAS / CNRS / Univ. Paris-Sud
# BSD License - see attached LICENSE file
# Author: Alexandre Boucaud <alexandre.boucaud@ias.u-psud.fr>

"""
parser.py
---------
Custom parser that wrap up the common argparse ArgumentParser

"""
from __future__ import absolute_import

import argparse


class ArgumentParserError(Exception):
    """Custom exception for the ThrowingArgumentParser"""
    pass


class ThrowingArgumentParser(argparse.ArgumentParser):
    """
    Wrapper around ArgumentParser to overwrite the raised error

    When an error occurs, this parser throws an ArgumentParserError
    instead of exiting the code and printing a very basic usage form.

    The ArgumentParserError exception can thus be caught to do
    something else like printing your own docstring.

    Reference
    ---------
    http://stackoverflow.com/a/14728477

    """
    def error(self, message):
        """Custom empty error"""
        raise ArgumentParserError(message)
